-- PMDG 737-700 FS2020 Adf Radio

-- Detlef von Reusner
-- V100

-- July 15 2022 - Initial Version



local gPropIllumination = user_prop_get(user_prop_add_enum("Illumination","Day,Night,Dial","Day","Select fix or left forward panel dial controlled illumination"))

local gBgDay = img_add_fullscreen("bg-day-v2.png")

-- these light control variables are automatically set from XPlane datarefs (msfs uncapable right now)
local gDaylight = .0 
local gInteriorLight = .0
local gScalesLight = .5



local gColorDisplay = string.format("#%02x%02x%02x", 246, 148, 49)


local gColorText = string.format("#%02x%02x%02x", 237, 237, 237)
local gButtonTextStyle = "font:arimo_regular.ttf; size:14; halign:center; color:" .. gColorText


-- ================================== global variables ============================

local gTimerTurningFreq = nil
local gTimerTurningTic = 0
local gStopTimerTurningFreq = 0
local gImgDialTone
local gAdfToneOn = true

local gImgButReleased
local gImgArrow
local gAdfActive = 0
local gAdfStandby = 0

-- ======================== image order needed for illumination ===============

local gLightMask = img_add_fullscreen("sun-mask.png")
local gBgNight = img_add_fullscreen("bg-night-v2.png")

local gTxtActiveP1 = txt_add("987", "font:digital-7-mono.ttf; size:27;color:"..gColorDisplay..";halign:left;", 65, 22, 60, 30)
local gTxtActiveP2 = txt_add("000", "font:digital-7-mono.ttf; size:27;color:"..gColorDisplay..";halign:left;", 124, 22, 60, 30)

local gTxtStandbyP1 = txt_add("328", "font:digital-7-mono.ttf; size:27;color:"..gColorDisplay..";halign:left;", 229, 22, 120, 30)
local gTxtStandbyP2 = txt_add("610", "font:digital-7-mono.ttf; size:27;color:"..gColorDisplay..";halign:left;", 288, 22, 120, 30)

local gImgDotActive = canvas_add(120, 42, 6, 6, function()
  _circle(0, 0, 3)
  _fill(gColorDisplay)
end)
local gImgDotStandby = canvas_add(285, 42, 6, 6, function()
  _circle(0, 0, 3)
  _fill(gColorDisplay)
end)


local gImgModeAdfActive = canvas_add(39, 23, 23, 12, function()
  _rect(0, 0, 26, 12)
  _fill(gColorDisplay)
  _txt("ADF", "font:arimo_bold.ttf; size:13;color:black;halign:left;", 0, 0) 
end)

local gImgModeAntActive = canvas_add(39, 34, 23, 12, function()
  _rect(0, 0, 26, 12)
  _fill(gColorDisplay)
  _txt("ANT", "font:arimo_bold.ttf; size:13;color:black;halign:left;", 0, 0) 
end)

local gImgModeAdfStandby = canvas_add(203, 23, 23, 12, function()
  _rect(0, 0, 26, 12)
  _fill(gColorDisplay)
  _txt("ADF", "font:arimo_bold.ttf; size:13;color:black;halign:left;", 0, 0) 
end)

local gImgModeAntStandby = canvas_add(203, 34, 23, 12, function()
  _rect(0, 0, 26, 12)
  _fill(gColorDisplay)
  _txt("ANT", "font:arimo_bold.ttf; size:13;color:black;halign:left;", 0, 0) 
end)

-- ================================== Frequency knob ====================

-- 818 ADF mode 0 adf, 100 ant OK
-- 819 mode test switch, not used
-- 820 tone switch 0 off, 100 on, OK
-- 821 tone test, not needed
-- 827 ADF TFR switch 100== pressed, OK

  -- "L:switch_822_73X", "number", -- inner
  -- "L:switch_823_73X", "number", -- middle
  -- "L:switch_824_73X", "number", -- outer

-- 822 adf innerknob, 0.8333333 steps -> display .5 steps
-- 823 middle selector, display 10 steps
-- 824 outer knob, display 100 steps

function cbTimerTurningFreq(count, limit)
  if count >= gStopTimerTurningFreq or count>=limit then
    timer_stop(gTimerTurningFreq)
    gTimerTurningFreq = nil
    gGetStandbyFreq = true
    fs2020_event("ROTOR_BRAKE", 82701) -- swap button (causes two swaps to get the standby freq from the active freq)
  else
    gTimerTurningTic = gTimerTurningTic+1
  end
end

function RefreshTurningTimer()
  if gTimerTurningFreq == nil then
    gTimerTurningTic = 0
    gStopTimerTurningFreq = 2
    gTimerTurningFreq = timer_start(0, 500, 20, cbTimerTurningFreq)
  else
    gStopTimerTurningFreq = gTimerTurningTic + 2
  end
end


function cbTurnOuterFreqKnob(direc)
  RefreshTurningTimer()
  if direc>0 then
    fs2020_event("ROTOR_BRAKE", 82307) -- adf for middle knob
    gAdfStandby = gAdfStandby + 10000
    ShowDisplays()
  else
    fs2020_event("ROTOR_BRAKE", 82308)
    gAdfStandby = gAdfStandby - 10000
    ShowDisplays()
  end
end
dial_add("outer-knob.png", 253, 84, 60, 60, cbTurnOuterFreqKnob)


function cbTurnInnerFreqKnob(direc)
  RefreshTurningTimer()
  if direc>0 then
    fs2020_event("ROTOR_BRAKE", 82207) -- adf
    gAdfStandby = gAdfStandby + 500
    ShowDisplays()
  else
    fs2020_event("ROTOR_BRAKE", 82208)
    gAdfStandby = gAdfStandby - 500
    ShowDisplays()
  end
end
dial_add("inner-knob.png", 262, 94, 41, 41, cbTurnInnerFreqKnob)

-- ============================= TONE knob (as button) ======================

function ShowAdfToneOnOff()
  if gAdfToneOn then
    img_rotate(gImgDialTone, 30, "LINEAR", 0.1)
  else
    img_rotate(gImgDialTone, -30, "LINEAR", 0.1)
  end
end

img_add("pointer-shadow.png", 156, 72, 70, 70)
gImgDialTone = img_add("pointer.png", 156, 72, 70, 70)
button_add(nil, nil, 156, 72, 70, 70, function() fs2020_event("ROTOR_BRAKE", 82001); ShowAdfToneOnOff() end)
fs2020_variable_subscribe("L:switch_820_73X", "number", function(v) gAdfToneOn = (v==100); ShowAdfToneOnOff() end)

-- ======================= Mode Adf Ant knob as button ===================

function ShowMode()
  if gModeAnt then
    img_rotate(gImgMode, 30, "LINEAR", 0.1)
    visible(gImgModeAdfActive, false)
    visible(gImgModeAntActive, true)
    visible(gImgModeAdfStandby, false)
    visible(gImgModeAntStandby, true)
  else
    img_rotate(gImgMode, -30, "LINEAR", 0.1)
    visible(gImgModeAdfActive, true)
    visible(gImgModeAntActive, false)
    visible(gImgModeAdfStandby, true)
    visible(gImgModeAntStandby, false)
  end
end

img_add("pointer-shadow.png", 67, 72, 70, 70)
gImgMode = img_add("pointer.png", 67, 72, 70, 70)
button_add(nil, nil, 67, 72, 70, 70, function() fs2020_event("ROTOR_BRAKE", 81801); ShowMode() end)
fs2020_variable_subscribe("L:switch_818_73X", "number", function(v) gModeAnt = (v==100); ShowMode() end)

-- ========================= Button functions ===========================================

function ButtonPressed()
  -- print("active: "..tostring(gAdfActive).."   standby: "..tostring(gAdfStandby))
  gAdfStandby = gAdfActive
  fs2020_event("ROTOR_BRAKE", 82701) -- TFR swap
  visible(gImgButReleased, false)
  move(gImgArrow, 66, 31)
  ShowDisplays()
end

function ButtonReleased()
  visible(gImgButReleased, true)
  move(gImgArrow, 66, 30)
end

img_add("button-pressed.png", 150, 28, 36, 18)
gImgButReleased = img_add("button-released.png", 150, 28, 36, 18)
gImgArrow = img_add("swap-arrow.png", 0, 0, 130, 15)
button_add(nil, nil, 140, 15, 60, 35, function() ButtonPressed() end, function() ButtonReleased() end)
ButtonReleased()

-- ============================ Displays ==============================



-- -- -- function MakeFrequencyString(v)
  -- -- -- local p1, p2
  -- -- -- v = (v+1)/1000000
  -- -- -- p1 = math.floor(v)
  -- -- -- p2 = math.floor((v-p1)*1000)
  -- -- -- return string.format("%03d", p1), string.format("%03d", p2)
-- -- -- end
function MakeFrequencyString(v)
  local p1, p2
  v = v/1000
  p1 = math.floor(v)
  p2 = math.floor((v-p1)*10)
  local a, b
  a = string.format("%04d", p1)
  b = string.format("%01d", p2)
  -- print("###"..a.."###"..b.."###")
  return a, b
end

function ShowDisplays()
  -- print("led 2 is "..tostring(gLed[2].is_on))
  -- print("led 3 is "..tostring(gLed[3].is_on))

  local p1, p2

  visible(gImgDotActive, true)
  visible(gImgDotStandby, true)
  p1, p2 = MakeFrequencyString(gAdfActive)
  txt_set(gTxtActiveP1, p1)
  txt_set(gTxtActiveP2, p2)

  p1, p2 = MakeFrequencyString(gAdfStandby)
  txt_set(gTxtStandbyP1, p1)
  txt_set(gTxtStandbyP2, p2)
end

function cbFreqChange(a)
  if gGetStandbyFreq then
    gAdfStandby = a
    gGetStandbyFreq = false
    fs2020_event("ROTOR_BRAKE", 82701) -- swap button
  else
    gAdfActive = a
  end
  ShowDisplays()
end


fs2020_variable_subscribe(
  "ADF ACTIVE FREQUENCY:1", "number",
  cbFreqChange)
  

function cbStandbyFreqChange(inner, middle, outer)
  RefreshTurningTimer()
end
  
fs2020_variable_subscribe(
  "L:switch_822_73X", "number", -- inner
  "L:switch_823_73X", "number", -- middle
  "L:switch_824_73X", "number", -- outer
  cbStandbyFreqChange)


-- ================== illumination ========================

function cbLightChange()
  -- print("daylight: " .. gDaylight .. "   scales: ".. gScalesLight)
  local daylight = var_cap(gDaylight, .2, gDaylight)
  opacity(gLightMask, 1-daylight)
  opacity(gBgNight, gScalesLight)
end

if gPropIllumination == "Day" then
  gDaylight = .99
  gScalesLight = 0
  cbLightChange()
elseif gPropIllumination == "Night" then
  gDaylight = 0
  gScalesLight = .5
  cbLightChange()
else
  fs2020_variable_subscribe("A:LIGHT POTENTIOMETER:2", "Number",  function(v) gDaylight = v; cbLightChange() end)
  fs2020_variable_subscribe("A:LIGHT POTENTIOMETER:3", "Number",  function(v) gScalesLight = v; cbLightChange() end)
end


